/* /////////////////////////////////////////////////////////////////////////////
 * File:        std/dtl/range/ranges.d
 *
 * Purpose:     Range base classes.
 *
 * Created      15th July 2004
 * Updated:     31st July 2004
 *
 * www:         http://www.synesis.com.au/software/
 *
 * Copyright (C) 2004 by Matthew Wilson
 *
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from the
 * use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not claim
 * that you wrote the original software. If you use this software in a product,
 * an acknowledgment in the product documentation would be appreciated but is
 * not required.
 *
 * 2. Altered source versions must be plainly marked as such, and must not be
 * misrepresented as being the original software.
 *
 * 3. This notice may not be removed or altered from any source distribution. 
 *
 * ////////////////////////////////////////////////////////////////////////// */


/** \file std/dtl/range/ranges.d Range base classes */

/* ////////////////////////////////////////////////////////////////////////// */

module std.dtl.range.ranges;

/* /////////////////////////////////////////////////////////////////////////////
 * Imports
 */

import std.dtl.common;
import std.dtl.range.categories;

/* ////////////////////////////////////////////////////////////////////////// */

template NotionalRange(V) { interface NotionalRange
	: public NotionalRangeTag
{
/// \name Member types
/// @{
public:
	alias	V					value_type;
	alias	NotionalRangeTag	range_category;
/// @}

/// \name Notional range methods
/// @{
public:
    bool		is_open();
    value_type	current();
    void		advance();
/// @}
}}

template SubscriptableRange(V) { interface SubscriptableRange
	: public NotionalRange!(V)
	, public SubscriptableRangeTag
{
/// \name Member types
/// @{
public:
	alias	NotionalRange!(V)			parent_class_type;
public:
	alias	V							value_type;
	alias	std.dtl.common.difference_type	difference_type;
	alias	std.dtl.common.index_type		index_type;
	alias	std.dtl.common.size_type		size_type;
	alias	RandomAccessRangeTag		range_category;
/// @}

/// \name Notional range methods
/// @{
public:
    bool		is_open();
    value_type	current();
    void		advance();
/// @}

/// \name Random access range methods
/// @{
public:
	size_type	length();
	value_type	opIndex(index_type index);
    void		advance(difference_type increment);
/// @}
}}


template RandomAccessRange(V) { interface RandomAccessRange
	: public SubscriptableRange!(V)
	, public RandomAccessRangeTag
{
}}

/* ////////////////////////////////////////////////////////////////////////// */
