/* /////////////////////////////////////////////////////////////////////////////
 * File:        type/traits.d
 *
 * Purpose:     Type traits
 *
 * Created      13th July 2004
 * Updated:     17th July 2004
 *
 * www:         http://www.synesis.com.au/software
 *              http://www.synsoft.org/
 *
 * Copyright (C) 2004 by Matthew Wilson and Synesis Software
 *
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from the
 * use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not claim
 * that you wrote the original software. If you use this software in a product,
 * an acknowledgment in the product documentation would be appreciated but is
 * not required.
 *
 * 2. Altered source versions must be plainly marked as such, and must not be
 * misrepresented as being the original software.
 *
 * 3. This notice may not be removed or altered from any source distribution. 
 *
 * ////////////////////////////////////////////////////////////////////////// */


/** \file type/traits.d Type traits */

/* ////////////////////////////////////////////////////////////////////////// */

module std.type.traits;

/* /////////////////////////////////////////////////////////////////////////////
 * Imports
 */

/* ////////////////////////////////////////////////////////////////////////// */

private template _isMaxValue(T) { bool _isMaxValue(T value)
{
	return value == T.max;
}}

private template _isMinValue(T) { bool _isMinValue(T value)
{
	return value == T.min;
}}

/* ////////////////////////////////////////////////////////////////////////// */

bool isMaxValue(bit value)		{	return _isMaxValue!(bit)(value);	}
bool isMaxValue(byte value)		{	return _isMaxValue!(byte)(value);	}
bool isMaxValue(ubyte value)	{	return _isMaxValue!(ubyte)(value);	}
bool isMaxValue(short value)	{	return _isMaxValue!(short)(value);	}
bool isMaxValue(ushort value)	{	return _isMaxValue!(ushort)(value);	}
bool isMaxValue(int value)		{	return _isMaxValue!(int)(value);	}
bool isMaxValue(uint value)		{	return _isMaxValue!(uint)(value);	}
bool isMaxValue(long value)		{	return _isMaxValue!(long)(value);	}
bool isMaxValue(ulong value)	{	return _isMaxValue!(ulong)(value);	}

bool isMaxValue(Object value)
{
	return false;
}

bool isMinValue(bit value)		{	return _isMinValue!(bit)(value);	}
bool isMinValue(byte value)		{	return _isMinValue!(byte)(value);	}
bool isMinValue(ubyte value)	{	return _isMinValue!(ubyte)(value);	}
bool isMinValue(short value)	{	return _isMinValue!(short)(value);	}
bool isMinValue(ushort value)	{	return _isMinValue!(ushort)(value);	}
bool isMinValue(int value)		{	return _isMinValue!(int)(value);	}
bool isMinValue(uint value)		{	return _isMinValue!(uint)(value);	}
bool isMinValue(long value)		{	return _isMinValue!(long)(value);	}
bool isMinValue(ulong value)	{	return _isMinValue!(ulong)(value);	}

bool isMinValue(Object value)
{
	return false;
}



/+
private template _isValue(T, alias P) { bool _isValue(T value)
{
	return value == T.P;
}}


bool isMinValue(bit value)		{	return _isValue!(bit, min)(value);		}
bool isMinValue(byte value)		{	return _isValue!(byte, min)(value);		}
bool isMinValue(ubyte value)	{	return _isValue!(ubyte, min)(value);	}
bool isMinValue(short value)	{	return _isValue!(short, min)(value);	}
bool isMinValue(ushort value)	{	return _isValue!(ushort, min)(value);	}
bool isMinValue(int value)		{	return _isValue!(int, min)(value);		}
bool isMinValue(uint value)		{	return _isValue!(uint, min)(value);		}
bool isMinValue(long value)		{	return _isValue!(long, min)(value);		}
bool isMinValue(ulong value)	{	return _isValue!(ulong, min)(value);	}

bool isMinValue(Object value)
{
	return false;
}
+/

/* ////////////////////////////////////////////////////////////////////////// */
